searchData={"items":[{"type":"module","title":"NimbleOwnership","doc":"Module that allows you to manage ownership of resources across processes.\n\nThe idea is that you can track ownership of terms (keys) across processes,\nand allow processes to use a key through processes that are already allowed.\n\n```mermaid\nflowchart LR\n  pidA[\"Process A\"]\n  pidB[\"Process B\"]\n  pidC[\"Process C\"]\n  res([\"Resource\"])\n\n  pidA -->|Owns| res\n  pidA -->|Allows| pidB\n  pidB -->|Can access| res\n  pidB -->|Allows| pidC\n  pidC -->|Can access| res\n```\n\nA typical use case for such a module is tracking resource ownership across processes\nin order to isolate access to resources in **test suites**. For example, the\n[Mox](https://hexdocs.pm/mox/Mox.html) library uses this module to track ownership\nof mocks across processes (in shared mode).","ref":"NimbleOwnership.html"},{"type":"module","title":"Usage - NimbleOwnership","doc":"To track ownership of resources, you need to start a `NimbleOwnership` server (a process),\nthrough `start_link/1` or `child_spec/1`.\n\nThen, you can allow a process access to a key through `allow/5`. You can then check\nif a PID can access the given key through `get_owner/3`.\n\n#","ref":"NimbleOwnership.html#module-usage"},{"type":"module","title":"Metadata - NimbleOwnership","doc":"You can store arbitrary metadata (`t:metadata/0`) alongside each \"allowance\", that is,\nalongside the relationship between a process and a key. This metadata is returned\ntogether with the owner PID when you call `get_owner/3`.","ref":"NimbleOwnership.html#module-metadata"},{"type":"function","title":"NimbleOwnership.allow/5","doc":"Allows `pid_to_allow` to use `key` through `owner_pid` (on the given `ownership_server`).\n\nUse this function when `owner_pid` is allowed access to `key`, and you want\nto also allow `pid_to_allow` to use `key`.\n\n`metadata` is an arbitrary term that you can use to store additional information\nabout the allowance. It is returned by `get_owner/3`.\n\nThis function returns an error when `pid_to_allow` is already allowed to use\n`key` via **another owner PID** that is not `owner_pid`.","ref":"NimbleOwnership.html#allow/5"},{"type":"function","title":"Examples - NimbleOwnership.allow/5","doc":"iex> pid = spawn(fn -> Process.sleep(:infinity) end)\n    iex> {:ok, server} = NimbleOwnership.start_link()\n    iex> NimbleOwnership.allow(server, self(), pid, :my_key, %{counter: 1})\n    :ok\n    iex> NimbleOwnership.get_owner(server, [pid], :my_key)\n    %{owner_pid: self(), metadata: %{counter: 1}}","ref":"NimbleOwnership.html#allow/5-examples"},{"type":"function","title":"NimbleOwnership.child_spec/1","doc":"Returns a specification to start this module under a supervisor.\n\nSee `Supervisor`.","ref":"NimbleOwnership.html#child_spec/1"},{"type":"function","title":"NimbleOwnership.get_owner/3","doc":"Retrieves the first PID in `callers` that is allowed to use `key` (on\nthe given `ownership_server`).\n\nSee `allow/5` for examples.","ref":"NimbleOwnership.html#get_owner/3"},{"type":"function","title":"NimbleOwnership.start_link/1","doc":"Starts an ownership server.","ref":"NimbleOwnership.html#start_link/1"},{"type":"function","title":"Options - NimbleOwnership.start_link/1","doc":"This function supports all the options supported by `GenServer.start_link/3`, namely:\n\n  * `:name`\n  * `:timeout`\n  * `:debug`\n  * `:spawn_opt`\n  * `:hibernate_after`","ref":"NimbleOwnership.html#start_link/1-options"},{"type":"type","title":"NimbleOwnership.key/0","doc":"Arbitrary key.","ref":"NimbleOwnership.html#t:key/0"},{"type":"type","title":"NimbleOwnership.metadata/0","doc":"Arbitrary metadata associated with an *allowance*.","ref":"NimbleOwnership.html#t:metadata/0"},{"type":"type","title":"NimbleOwnership.owner_info/0","doc":"Information about the owner of a key returned by `get_owner/3`.","ref":"NimbleOwnership.html#t:owner_info/0"},{"type":"type","title":"NimbleOwnership.server/0","doc":"Ownership server.","ref":"NimbleOwnership.html#t:server/0"},{"type":"exception","title":"NimbleOwnership.Error","doc":"Exception struct returned by `NimbleOwnership` functions.","ref":"NimbleOwnership.Error.html"},{"type":"type","title":"NimbleOwnership.Error.t/0","doc":"","ref":"NimbleOwnership.Error.html#t:t/0"}],"content_type":"text/markdown"}